/*
******************************************************************************
* File Name    : printfUart1_DMA_Driver.c
* Created On   : 4 Mar. 2022
* Description  :
* Author	     : R J Garnett - Admin
******************************************************************************
*/

#include "FreeRTOS.h"
#include "task.h"
#include "cmsis_os.h"

typedef enum
{
	PRINTF_OK,
	PRINTF_BUFFER_FULL,
	PRINTF_HAL_DRIVER_ERROR,
	PRINTF_INIT_FAILED,

}status_t_enum ;


TaskHandle_t 	printTaskHandle;
StackType_t 	printTaskStack[1024];
StaticTask_t 	printTaskBuffer;

static void printTask(void * argument);
static status_t_enum startPrintTask(void);

/*************************************************************************************/
void initPrintfUart1(void)
{
  /* Start the printing thread */
  startPrintTask();
}

/*************************************************************************************/
int printfdma(const char *format, ...)
{
	int nResult = 0;

 	/* Kick off the uart */
	xTaskNotifyGive(printTaskHandle);

  return nResult;
}

/*
 ***************************************************************************
 * FREE RTOS TASKS AND SEMAPHORE
 *
 ***************************************************************************
 */


static status_t_enum startPrintTask(void)
{
	printTaskHandle 	 = xTaskCreateStatic(printTask,
																				 "printTask",
																				 1024,
																				 NULL,
																				 osPriorityNormal1,
																				 printTaskStack,
																				 &printTaskBuffer);

	if(printTaskHandle == NULL)
	{
		return PRINTF_INIT_FAILED;
	}
	return PRINTF_OK;
}


/*************************************************************************************/
static void printTask(void * argument)
{
	static uint32_t thread_notification;
	static int k = 0;

	/* call the thread notification to set the counter to zero so it blocks */
	thread_notification = ulTaskNotifyTake(pdTRUE, 0);

	while(1)
	{
    thread_notification = ulTaskNotifyTake(pdTRUE, portMAX_DELAY);
    if(thread_notification)
    {
			k++;
		} /* end of thread notification true */
	}
	vTaskDelay(5);
}

